"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var baseplatform_1 = require("../baseplatform");
var emu_1 = require("../emu");
var util_1 = require("../util");
var audio_1 = require("../audio");
var tms9918a_1 = require("../video/tms9918a");
// http://www.colecovision.eu/ColecoVision/development/tutorial1.shtml
// http://www.colecovision.eu/ColecoVision/development/libcv.shtml
// http://www.kernelcrash.com/blog/recreating-the-colecovision/2016/01/27/
// http://www.atarihq.com/danb/files/CV-Tech.txt
// http://www.atarihq.com/danb/files/CV-Sound.txt
// http://www.colecoboxart.com/faq/FAQ05.htm
// http://www.theadamresource.com/manuals/technical/Jeffcoleco.html
// http://bifi.msxnet.org/msxnet//tech/tms9918a.txt
// http://www.colecovision.dk/tools.htm?refreshed
// http://www.theadamresource.com/manuals/technical/ColecoVision%20Coding%20Guide.pdf
// http://www.unige.ch/medecine/nouspikel/ti99/tms9918a.htm
// http://map.grauw.nl/articles/vdp_tut.php
// http://www.msxcomputermagazine.nl/mccw/91/msx1demos1/en.html
// http://www.segordon.com/colecovision.php
// http://samdal.com/svvideo.htm
// https://github.com/tursilion/convert9918
// http://www.harmlesslion.com/cgi-bin/showprog.cgi?ColecoVision
exports.ColecoVision_PRESETS = [
    { id: 'text.c', name: 'Text Mode' },
    { id: 'hello.c', name: 'Scrolling Text' },
    { id: 'text32.c', name: '32-Column Color Text' },
    { id: 'stars.c', name: 'Scrolling Starfield' },
    { id: 'cursorsmooth.c', name: 'Moving Cursor' },
    { id: 'simplemusic.c', name: 'Simple Music' },
    { id: 'musicplayer.c', name: 'Multivoice Music' },
    { id: 'mode2bitmap.c', name: 'Mode 2 Bitmap' },
    { id: 'mode2compressed.c', name: 'Mode 2 Bitmap (LZG)' },
    { id: 'lines.c', name: 'Mode 2 Lines' },
    { id: 'multicolor.c', name: 'Multicolor Mode' },
    { id: 'siegegame.c', name: 'Siege Game' },
    { id: 'shoot.c', name: 'Solarian Game' },
    { id: 'climber.c', name: 'Platform Game' },
];
var COLECOVISION_KEYCODE_MAP = emu_1.makeKeycodeMap([
    [emu_1.Keys.VK_UP, 0, 0x1],
    [emu_1.Keys.VK_DOWN, 0, 0x4],
    [emu_1.Keys.VK_LEFT, 0, 0x8],
    [emu_1.Keys.VK_RIGHT, 0, 0x2],
    [emu_1.Keys.VK_SPACE, 0, 0x40],
    [emu_1.Keys.VK_CONTROL, 1, 0x40],
    [emu_1.Keys.VK_W, 2, 0x1],
    [emu_1.Keys.VK_S, 2, 0x4],
    [emu_1.Keys.VK_A, 2, 0x8],
    [emu_1.Keys.VK_D, 2, 0x2],
    [emu_1.Keys.VK_Z, 2, 0x40],
    [emu_1.Keys.VK_X, 3, 0x40],
]);
/// standard emulator
var _ColecoVisionPlatform = function (mainElement) {
    var cpuFrequency = 3579545; // MHz
    var canvasWidth = 304;
    var numTotalScanlines = 262;
    var numVisibleScanlines = 240;
    var cpuCyclesPerLine = Math.round(cpuFrequency / 60 / numTotalScanlines);
    var cpu, ram, membus, iobus, rom, bios;
    var video, vdp, timer;
    var audio, psg;
    var inputs = new Uint8Array(4);
    var keypadMode = false;
    var ColecoVisionPlatform = /** @class */ (function (_super) {
        __extends(ColecoVisionPlatform, _super);
        function ColecoVisionPlatform() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ColecoVisionPlatform.prototype.getPresets = function () { return exports.ColecoVision_PRESETS; };
        ColecoVisionPlatform.prototype.start = function () {
            ram = new emu_1.RAM(1024);
            bios = new util_1.lzgmini().decode(util_1.stringToByteArray(atob(COLECO_BIOS_LZG)));
            membus = {
                read: emu_1.newAddressDecoder([
                    [0x0000, 0x1fff, 0x1fff, function (a) { return bios ? bios[a] : 0; }],
                    [0x6000, 0x7fff, 0x3ff, function (a) { return ram.mem[a]; }],
                    [0x8000, 0xffff, 0x7fff, function (a) { return rom ? rom[a] : 0; }],
                ]),
                write: emu_1.newAddressDecoder([
                    [0x6000, 0x7fff, 0x3ff, function (a, v) { ram.mem[a] = v; }],
                ]),
                isContended: function () { return false; },
            };
            iobus = {
                read: function (addr) {
                    addr &= 0xff;
                    //console.log('IO read', hex(addr,4));
                    switch (addr) {
                        case 0xfc: return inputs[keypadMode ? 1 : 0] ^ 0xff;
                        case 0xff: return inputs[keypadMode ? 3 : 2] ^ 0xff;
                    }
                    if (addr >= 0xa0 && addr <= 0xbf) {
                        if (addr & 1)
                            return vdp.readStatus();
                        else
                            return vdp.readData();
                    }
                    return 0;
                },
                write: function (addr, val) {
                    addr &= 0xff;
                    val &= 0xff;
                    //console.log('IO write', hex(addr,4), hex(val,2));
                    switch (addr >> 4) {
                        case 0x8:
                        case 0x9:
                            keypadMode = true;
                            break;
                        case 0xc:
                        case 0xd:
                            keypadMode = false;
                            break;
                        case 0xa:
                        case 0xb:
                            if (addr & 1)
                                return vdp.writeAddress(val);
                            else
                                return vdp.writeData(val);
                        case 0xf:
                            psg.setData(val);
                            break;
                    }
                }
            };
            cpu = this.newCPU(membus, iobus);
            video = new emu_1.RasterVideo(mainElement, canvasWidth, numVisibleScanlines, { overscan: true });
            video.create();
            audio = new audio_1.MasterAudio();
            psg = new audio_1.SN76489_Audio(audio);
            var cru = {
                setVDPInterrupt: function (b) {
                    if (b) {
                        cpu.nonMaskableInterrupt();
                    }
                    else {
                        // TODO: reset interrupt?
                    }
                }
            };
            vdp = new tms9918a_1.TMS9918A(video.getFrameData(), cru, true); // true = 4 sprites/line
            emu_1.setKeyboardFromMap(video, inputs, COLECOVISION_KEYCODE_MAP);
            timer = new emu_1.AnimationTimer(60, this.nextFrame.bind(this));
        };
        ColecoVisionPlatform.prototype.readAddress = function (addr) {
            return membus.read(addr);
        };
        ColecoVisionPlatform.prototype.advance = function (novideo) {
            for (var sl = 0; sl < numTotalScanlines; sl++) {
                this.runCPU(cpu, cpuCyclesPerLine);
                vdp.drawScanline(sl);
            }
            video.updateFrame();
        };
        ColecoVisionPlatform.prototype.loadROM = function (title, data) {
            rom = emu_1.padBytes(data, 0x8000);
            this.reset();
        };
        ColecoVisionPlatform.prototype.loadState = function (state) {
            cpu.loadState(state.c);
            ram.mem.set(state.b);
            vdp.restoreState(state.vdp);
            keypadMode = state.kpm;
            inputs.set(state.in);
        };
        ColecoVisionPlatform.prototype.saveState = function () {
            return {
                c: this.getCPUState(),
                b: ram.mem.slice(0),
                vdp: vdp.getState(),
                kpm: keypadMode,
                in: inputs.slice(0),
            };
        };
        ColecoVisionPlatform.prototype.loadControlsState = function (state) {
            inputs.set(state.in);
        };
        ColecoVisionPlatform.prototype.saveControlsState = function () {
            return {
                in: inputs.slice(0)
            };
        };
        ColecoVisionPlatform.prototype.getCPUState = function () {
            return cpu.saveState();
        };
        ColecoVisionPlatform.prototype.isRunning = function () {
            return timer && timer.isRunning();
        };
        ColecoVisionPlatform.prototype.pause = function () {
            timer.stop();
            audio.stop();
        };
        ColecoVisionPlatform.prototype.resume = function () {
            timer.start();
            audio.start();
        };
        ColecoVisionPlatform.prototype.reset = function () {
            cpu.reset();
            cpu.setTstates(0);
            vdp.reset();
            psg.reset();
        };
        ColecoVisionPlatform.prototype.getDebugCategories = function () {
            return _super.prototype.getDebugCategories.call(this).concat(['VDP']);
        };
        ColecoVisionPlatform.prototype.getDebugInfo = function (category, state) {
            switch (category) {
                case 'VDP': return this.vdpStateToLongString(state.vdp);
                default: return _super.prototype.getDebugInfo.call(this, category, state);
            }
        };
        ColecoVisionPlatform.prototype.vdpStateToLongString = function (ppu) {
            return vdp.getRegsString();
        };
        return ColecoVisionPlatform;
    }(baseplatform_1.BaseZ80Platform));
    return new ColecoVisionPlatform();
};
var COLECO_BIOS_LZG = "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";
/// MAME support
var ColecoVisionMAMEPlatform = /** @class */ (function (_super) {
    __extends(ColecoVisionMAMEPlatform, _super);
    function ColecoVisionMAMEPlatform() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.getToolForFilename = baseplatform_1.getToolForFilename_z80;
        return _this;
    }
    ColecoVisionMAMEPlatform.prototype.start = function () {
        this.startModule(this.mainElement, {
            jsfile: 'mamecoleco.js',
            cfgfile: 'coleco.cfg',
            biosfile: 'coleco/313 10031-4005 73108a.u2',
            driver: 'coleco',
            width: 280 * 2,
            height: 216 * 2,
            romfn: '/emulator/cart.rom',
            romsize: 0x8000,
            preInit: function (_self) {
            },
        });
    };
    ColecoVisionMAMEPlatform.prototype.loadROM = function (title, data) {
        this.loadROMFile(data);
        this.loadRegion(":coleco_cart:rom", data);
    };
    ColecoVisionMAMEPlatform.prototype.getPresets = function () { return exports.ColecoVision_PRESETS; };
    ColecoVisionMAMEPlatform.prototype.getDefaultExtension = function () { return ".c"; };
    ;
    return ColecoVisionMAMEPlatform;
}(baseplatform_1.BaseMAMEPlatform));
///
emu_1.PLATFORMS['coleco.mame'] = ColecoVisionMAMEPlatform;
emu_1.PLATFORMS['coleco'] = _ColecoVisionPlatform;
//# sourceMappingURL=coleco.js.map