"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var baseplatform_1 = require("../baseplatform");
var emu_1 = require("../emu");
var util_1 = require("../util");
var audio_1 = require("../audio");
var tms9918a_1 = require("../video/tms9918a");
// http://www.smspower.org/Development/Index
// http://www.smspower.org/uploads/Development/sg1000.txt
// http://www.smspower.org/uploads/Development/richard.txt
// http://www.smspower.org/uploads/Development/msvdp-20021112.txt
// http://www.smspower.org/uploads/Development/SN76489-20030421.txt
// TODO: merge w/ coleco
exports.SG1000_PRESETS = [
    { id: 'text.c', name: 'Text Mode' },
    { id: 'hello.c', name: 'Scrolling Text' },
    { id: 'text32.c', name: '32-Column Color Text' },
    { id: 'stars.c', name: 'Scrolling Starfield' },
    { id: 'cursorsmooth.c', name: 'Moving Cursor' },
    { id: 'simplemusic.c', name: 'Simple Music' },
    { id: 'musicplayer.c', name: 'Multivoice Music' },
    { id: 'mode2bitmap.c', name: 'Mode 2 Bitmap' },
    { id: 'mode2compressed.c', name: 'Mode 2 Bitmap (LZG)' },
    { id: 'lines.c', name: 'Mode 2 Lines' },
    { id: 'multicolor.c', name: 'Multicolor Mode' },
    { id: 'siegegame.c', name: 'Siege Game' },
    { id: 'shoot.c', name: 'Solarian Game' },
    { id: 'climber.c', name: 'Platform Game' },
];
var SG1000_KEYCODE_MAP = emu_1.makeKeycodeMap([
    [emu_1.Keys.VK_UP, 0, 0x1],
    [emu_1.Keys.VK_DOWN, 0, 0x2],
    [emu_1.Keys.VK_LEFT, 0, 0x4],
    [emu_1.Keys.VK_RIGHT, 0, 0x8],
    [emu_1.Keys.VK_SPACE, 0, 0x10],
    [emu_1.Keys.VK_CONTROL, 0, 0x20],
    [emu_1.Keys.VK_R, 0, 0x40],
    [emu_1.Keys.VK_F, 0, 0x80],
    [emu_1.Keys.VK_D, 1, 0x1],
    [emu_1.Keys.VK_G, 1, 0x2],
    [emu_1.Keys.VK_A, 1, 0x4],
    [emu_1.Keys.VK_S, 1, 0x8],
    [emu_1.Keys.VK_1, 1, 0x10],
]);
var SG1000Platform = /** @class */ (function (_super) {
    __extends(SG1000Platform, _super);
    function SG1000Platform() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.cpuFrequency = 3579545; // MHz
        _this.canvasWidth = 304;
        _this.numTotalScanlines = 262;
        _this.numVisibleScanlines = 240;
        _this.defaultROMSize = 0xc000;
        return _this;
    }
    SG1000Platform.prototype.getPresets = function () { return exports.SG1000_PRESETS; };
    SG1000Platform.prototype.getKeyboardMap = function () { return SG1000_KEYCODE_MAP; };
    SG1000Platform.prototype.getVideoOptions = function () { return { overscan: true }; };
    SG1000Platform.prototype.newRAM = function () {
        return new Uint8Array(0x400);
    };
    SG1000Platform.prototype.newMembus = function () {
        var _this = this;
        return {
            read: emu_1.newAddressDecoder([
                [0xc000, 0xffff, 0x3ff, function (a) { return _this.ram[a]; }],
                [0x0000, 0xbfff, 0xffff, function (a) { return _this.rom[a]; }],
            ]),
            write: emu_1.newAddressDecoder([
                [0xc000, 0xffff, 0x3ff, function (a, v) { _this.ram[a] = v; }],
            ]),
            isContended: function () { return false; },
        };
    };
    SG1000Platform.prototype.getVCounter = function () { return 0; };
    SG1000Platform.prototype.getHCounter = function () { return 0; };
    SG1000Platform.prototype.setMemoryControl = function (v) { };
    SG1000Platform.prototype.setIOPortControl = function (v) { };
    SG1000Platform.prototype.newIOBus = function () {
        var _this = this;
        return {
            read: function (addr) {
                addr &= 0xff;
                //console.log('IO read', hex(addr,4));
                switch (addr & 0xc1) {
                    case 0x40: return _this.getVCounter();
                    case 0x41: return _this.getHCounter();
                    case 0x80: return _this.vdp.readData();
                    case 0x81: return _this.vdp.readStatus();
                    case 0xc0: return _this.inputs[0] ^ 0xff;
                    case 0xc1: return _this.inputs[1] ^ 0xff;
                }
                return 0;
            },
            write: function (addr, val) {
                addr &= 0xff;
                val &= 0xff;
                //console.log('IO write', hex(addr,4), hex(val,2));
                switch (addr & 0xc1) {
                    case 0x00: return _this.setMemoryControl(val);
                    case 0x01: return _this.setIOPortControl(val);
                    case 0x40:
                    case 0x41: return _this.psg.setData(val);
                    case 0x80: return _this.vdp.writeData(val);
                    case 0x81: return _this.vdp.writeAddress(val);
                }
            }
        };
    };
    SG1000Platform.prototype.newVDP = function (frameData, cru, flicker) {
        return new tms9918a_1.TMS9918A(frameData, cru, flicker);
    };
    SG1000Platform.prototype.start = function () {
        var _this = this;
        _super.prototype.start.call(this);
        this.audio = new audio_1.MasterAudio();
        this.psg = new audio_1.SN76489_Audio(this.audio);
        var cru = {
            setVDPInterrupt: function (b) {
                if (b) {
                    _this.cpu.nonMaskableInterrupt();
                }
                else {
                    // TODO: reset interrupt?
                }
            }
        };
        this.vdp = this.newVDP(this.video.getFrameData(), cru, true); // true = 4 sprites/line
    };
    SG1000Platform.prototype.startScanline = function (sl) {
    };
    SG1000Platform.prototype.drawScanline = function (sl) {
        this.vdp.drawScanline(sl);
    };
    SG1000Platform.prototype.loadState = function (state) {
        _super.prototype.loadState.call(this, state);
        this.vdp.restoreState(state['vdp']);
    };
    SG1000Platform.prototype.saveState = function () {
        var state = _super.prototype.saveState.call(this);
        state['vdp'] = this.vdp.getState();
        return state;
    };
    SG1000Platform.prototype.reset = function () {
        _super.prototype.reset.call(this);
        this.vdp.reset();
        this.psg.reset();
    };
    SG1000Platform.prototype.getDebugCategories = function () {
        return _super.prototype.getDebugCategories.call(this).concat(['VDP']);
    };
    SG1000Platform.prototype.getDebugInfo = function (category, state) {
        switch (category) {
            case 'VDP': return this.vdpStateToLongString(state.vdp);
            default: return _super.prototype.getDebugInfo.call(this, category, state);
        }
    };
    SG1000Platform.prototype.vdpStateToLongString = function (ppu) {
        return this.vdp.getRegsString();
    };
    return SG1000Platform;
}(baseplatform_1.BasicZ80ScanlinePlatform));
///
var SMSPlatform = /** @class */ (function (_super) {
    __extends(SMSPlatform, _super);
    function SMSPlatform() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.cartram = new Uint8Array(0);
        _this.pagingRegisters = new Uint8Array(4);
        _this.latchedHCounter = 0;
        _this.ioControlFlags = 0;
        return _this;
    }
    // TODO: hide bottom scanlines
    SMSPlatform.prototype.reset = function () {
        _super.prototype.reset.call(this);
        this.pagingRegisters.set([0, 0, 1, 2]);
    };
    SMSPlatform.prototype.newVDP = function (frameData, cru, flicker) {
        return new tms9918a_1.SMSVDP(frameData, cru, flicker);
    };
    SMSPlatform.prototype.getVCounter = function () {
        var y = this.currentScanline;
        return (y <= 0xda) ? (y) : (y - 6);
    };
    SMSPlatform.prototype.getHCounter = function () {
        return this.latchedHCounter;
    };
    SMSPlatform.prototype.computeHCounter = function () {
        var t0 = this.startLineTstates;
        var t1 = this.cpu.getTstates();
        return (t1 - t0) & 0xff; // TODO
    };
    SMSPlatform.prototype.setIOPortControl = function (v) {
        if ((v ^ this.ioControlFlags) & 0xa0) { // either joystick TH pin
            this.latchedHCounter = this.computeHCounter();
            //console.log("H:"+hex(this.latchedHCounter)+" V:"+hex(this.getVCounter()));
        }
        this.ioControlFlags = v;
    };
    SMSPlatform.prototype.newRAM = function () {
        return new Uint8Array(0x2000);
    };
    SMSPlatform.prototype.getPagedROM = function (a, reg) {
        //if (!(a&0xff)) console.log(hex(a), reg, this.pagingRegisters[reg], this.romPageMask);
        return this.rom[a + ((this.pagingRegisters[reg] & this.romPageMask) << 14)]; // * $4000
    };
    SMSPlatform.prototype.newMembus = function () {
        var _this = this;
        return {
            read: emu_1.newAddressDecoder([
                [0xc000, 0xffff, 0x1fff, function (a) { return _this.ram[a]; }],
                [0x0000, 0x03ff, 0x3ff, function (a) { return _this.rom[a]; }],
                [0x0400, 0x3fff, 0x3fff, function (a) { return _this.getPagedROM(a, 1); }],
                [0x4000, 0x7fff, 0x3fff, function (a) { return _this.getPagedROM(a, 2); }],
                [0x8000, 0xbfff, 0x3fff, function (a) {
                        var reg0 = _this.pagingRegisters[0]; // RAM select?
                        if (reg0 & 0x8) {
                            return _this.cartram[(reg0 & 0x4) ? a + 0x4000 : a];
                        }
                        else {
                            return _this.getPagedROM(a, 3);
                        }
                    }],
            ]),
            write: emu_1.newAddressDecoder([
                [0xc000, 0xfffb, 0x1fff, function (a, v) {
                        _this.ram[a] = v;
                    }],
                [0xfffc, 0xffff, 0x3, function (a, v) {
                        _this.pagingRegisters[a] = v;
                        _this.ram[a + 0x1ffc] = v;
                    }],
                [0x8000, 0xbfff, 0x3fff, function (a, v) {
                        var reg0 = _this.pagingRegisters[0]; // RAM select?
                        if (reg0 & 0x8) {
                            if (_this.cartram.length == 0)
                                _this.cartram = new Uint8Array(0x8000); // create cartridge RAM lazily
                            _this.cartram[(reg0 & 0x4) ? a + 0x4000 : a] = v;
                        }
                    }],
            ]),
            isContended: function () { return false; },
        };
    };
    SMSPlatform.prototype.loadROM = function (title, data) {
        if (data.length <= 0xc000) {
            this.rom = emu_1.padBytes(data, 0xc000);
            this.romPageMask = 3; // only pages 0, 1, 2
        }
        else {
            switch (data.length) {
                case 0x10000:
                case 0x20000:
                case 0x40000:
                case 0x80000:
                    this.rom = data;
                    this.romPageMask = (data.length >> 14) - 1; // div $4000
                    break;
                default:
                    throw "Unknown rom size: $" + util_1.hex(data.length);
            }
        }
        //console.log("romPageMask: " + hex(this.romPageMask));
        this.reset();
    };
    SMSPlatform.prototype.loadState = function (state) {
        _super.prototype.loadState.call(this, state);
        this.pagingRegisters.set(state.pr);
        this.cartram.set(state.cr);
        this.latchedHCounter = state.lhc;
        this.ioControlFlags = state.iocf;
    };
    SMSPlatform.prototype.saveState = function () {
        var state = _super.prototype.saveState.call(this);
        state['pr'] = this.pagingRegisters.slice(0);
        state['cr'] = this.cartram.slice(0);
        state['lhc'] = this.latchedHCounter;
        state['iocf'] = this.ioControlFlags;
        return state;
    };
    SMSPlatform.prototype.getDebugInfo = function (category, state) {
        switch (category) {
            case 'CPU':
                return _super.prototype.getDebugInfo.call(this, category, state) +
                    "\nBank Regs: " + this.pagingRegisters + "\n";
            default: return _super.prototype.getDebugInfo.call(this, category, state);
        }
    };
    return SMSPlatform;
}(SG1000Platform));
///
emu_1.PLATFORMS['sms-sg1000-libcv'] = SG1000Platform;
emu_1.PLATFORMS['sms-sms-libcv'] = SMSPlatform;
//# sourceMappingURL=sms.js.map