"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var workertypes_1 = require("./workertypes");
var util_1 = require("./util");
var CodeProject = /** @class */ (function () {
    function CodeProject(worker, platform_id, platform, store) {
        var _this = this;
        this.filedata = {};
        this.pendingWorkerMessages = 0;
        this.tools_preloaded = {};
        this.isCompiling = false;
        this.worker = worker;
        this.platform_id = platform_id;
        this.platform = platform;
        this.store = store;
        worker.onmessage = function (e) {
            _this.receiveWorkerMessage(e.data);
        };
    }
    CodeProject.prototype.receiveWorkerMessage = function (data) {
        var notfinal = this.pendingWorkerMessages > 1;
        if (notfinal) {
            this.sendBuild();
            this.pendingWorkerMessages = 1;
        }
        else {
            if (this.callbackBuildStatus)
                this.callbackBuildStatus(false);
            if (!this.isCompiling) {
                console.log(this.pendingWorkerMessages);
                console.trace();
            } // debug compile problems
            this.isCompiling = false;
            this.pendingWorkerMessages = 0;
        }
        if (data && !data.unchanged) {
            this.processBuildResult(data);
            if (this.callbackBuildResult)
                this.callbackBuildResult(data); // call with data when changed
        }
    };
    CodeProject.prototype.preloadWorker = function (path) {
        var tool = this.platform.getToolForFilename(path);
        if (tool && !this.tools_preloaded[tool]) {
            this.worker.postMessage({ preload: tool, platform: this.platform_id });
            this.tools_preloaded[tool] = true;
        }
    };
    CodeProject.prototype.pushAllFiles = function (files, fn) {
        // look for local and preset files
        files.push('local/' + fn);
        files.push(fn);
        // look for files in current (main file) folder
        var dir = util_1.getFolderForPath(this.mainpath);
        if (dir.length > 0 && dir != 'local') // TODO
            files.push(dir + '/' + fn);
    };
    CodeProject.prototype.parseIncludeDependencies = function (text) {
        var files = [];
        var m;
        if (this.platform_id.startsWith('verilog')) {
            // include verilog includes
            var re1 = /^\s*(`include|[.]include)\s+"(.+?)"/gmi;
            while (m = re1.exec(text)) {
                this.pushAllFiles(files, m[2]);
            }
            // include .arch (json) statements
            var re2 = /^\s*([.]arch)\s+(\w+)/gmi;
            while (m = re2.exec(text)) {
                this.pushAllFiles(files, m[2] + ".json");
            }
            // include $readmem[bh] (TODO)
            var re3 = /\b\$readmem[bh]\("(.+?)"/gmi;
            while (m = re3.exec(text)) {
                this.pushAllFiles(files, m[2]);
            }
        }
        else {
            // for .asm -- [.]include "file"
            // for .c -- #include "file"
            var re2 = /^\s+([.#]?include|incbin)\s+"(.+?)"/gmi;
            while (m = re2.exec(text)) {
                this.pushAllFiles(files, m[2]);
            }
        }
        return files;
    };
    CodeProject.prototype.parseLinkDependencies = function (text) {
        var files = [];
        var m;
        if (this.platform_id.startsWith('verilog')) {
            //
        }
        else {
            // for .c -- //#link "file" (or ;link or #link)
            var re = /^\s*([;#]|[/][/][#])link\s+"(.+?)"/gm;
            while (m = re.exec(text)) {
                this.pushAllFiles(files, m[2]);
            }
        }
        return files;
    };
    CodeProject.prototype.loadFileDependencies = function (text, callback) {
        var includes = this.parseIncludeDependencies(text);
        var linkfiles = this.parseLinkDependencies(text);
        var allfiles = includes.concat(linkfiles);
        this.loadFiles(allfiles, function (err, result) {
            // set 'link' property on files that are link dependencies (must match filename)
            if (result)
                for (var _i = 0, result_1 = result; _i < result_1.length; _i++) {
                    var dep = result_1[_i];
                    dep.link = linkfiles.indexOf(dep.path) >= 0;
                }
            callback(err, result);
        });
    };
    CodeProject.prototype.okToSend = function () {
        return this.pendingWorkerMessages++ == 0;
    };
    CodeProject.prototype.updateFileInStore = function (path, text) {
        // protect against accidential whole-file deletion
        if (text.trim && text.trim().length) {
            this.store.setItem(path, text);
        }
    };
    // TODO: test duplicate files, local paths mixed with presets
    CodeProject.prototype.buildWorkerMessage = function (depends) {
        this.preloadWorker(this.mainpath);
        var msg = { updates: [], buildsteps: [] };
        // TODO: add preproc directive for __MAINFILE__
        var mainfilename = util_1.getFilenameForPath(this.mainpath);
        var maintext = this.getFile(this.mainpath);
        var depfiles = [];
        msg.updates.push({ path: mainfilename, data: maintext });
        for (var _i = 0, depends_1 = depends; _i < depends_1.length; _i++) {
            var dep = depends_1[_i];
            if (!dep.link) {
                msg.updates.push({ path: dep.filename, data: dep.data });
                depfiles.push(dep.filename);
            }
        }
        msg.buildsteps.push({ path: mainfilename, files: [mainfilename].concat(depfiles), platform: this.platform_id, tool: this.platform.getToolForFilename(this.mainpath), mainfile: true });
        for (var _a = 0, depends_2 = depends; _a < depends_2.length; _a++) {
            var dep = depends_2[_a];
            if (dep.data && dep.link) {
                this.preloadWorker(dep.filename);
                msg.updates.push({ path: dep.filename, data: dep.data });
                msg.buildsteps.push({ path: dep.filename, files: [dep.filename].concat(depfiles), platform: this.platform_id, tool: this.platform.getToolForFilename(dep.path) });
            }
        }
        return msg;
    };
    // TODO: get local file as well as presets?
    CodeProject.prototype.loadFiles = function (paths, callback) {
        var _this = this;
        var result = [];
        function addResult(path, data) {
            result.push({
                path: path,
                filename: util_1.getFilenameForPath(path),
                link: true,
                data: data
            });
        }
        var loadNext = function () {
            var path = paths.shift();
            if (!path) {
                // finished loading all files; return result
                callback(null, result);
            }
            else {
                // look in cache
                if (path in _this.filedata) { // found in cache?
                    var data = _this.filedata[path];
                    if (data)
                        addResult(path, data);
                    loadNext();
                }
                else {
                    // look in store
                    _this.store.getItem(path, function (err, value) {
                        if (err) { // err fetching from store
                            callback(err, result);
                        }
                        else if (value) { // found in store?
                            _this.filedata[path] = value; // do not update store, just cache
                            addResult(path, value);
                            loadNext();
                        }
                        else if (!path.startsWith("local/")) {
                            // don't load local/
                            // found on remote fetch?
                            var preset_id = _this.platform_id;
                            preset_id = preset_id.replace(/[.]\w+/, ''); // remove .suffix from preset name
                            var webpath = "presets/" + preset_id + "/" + path;
                            if (_this.platform_id.startsWith('vcs') && path.indexOf('.') <= 0)
                                webpath += ".a"; // legacy stuff
                            _this.callbackGetRemote(webpath, function (text) {
                                console.log("GET", webpath, text.length, 'bytes');
                                _this.filedata[path] = text; // do not update store, just cache
                                addResult(path, text);
                                loadNext();
                            }, 'text')
                                .fail(function (err) {
                                console.log("Could not load preset", path, err.status);
                                // only cache result if status is 404 (not found)
                                if (err.status && err.status == 404)
                                    _this.filedata[path] = null; // mark cache entry as invalid
                                loadNext();
                            });
                        }
                        else {
                            // not gonna find it, keep going
                            loadNext();
                        }
                    });
                }
            }
        };
        loadNext(); // load first file
    };
    CodeProject.prototype.getFile = function (path) {
        return this.filedata[path];
    };
    // TODO: purge files not included in latest build?
    CodeProject.prototype.iterateFiles = function (callback) {
        for (var path in this.filedata) {
            callback(path, this.getFile(path));
        }
    };
    CodeProject.prototype.sendBuild = function () {
        var _this = this;
        if (!this.mainpath)
            throw "need to call setMainFile first";
        var maindata = this.getFile(this.mainpath);
        // if binary blob, just return it as ROM
        if (maindata instanceof Uint8Array) {
            this.isCompiling = true;
            this.receiveWorkerMessage({
                output: maindata,
                errors: [],
                listings: null,
                symbolmap: null,
                params: {}
            });
            return;
        }
        // otherwise, make it a string
        var text = typeof maindata === "string" ? maindata : '';
        // TODO: load dependencies of non-main files
        this.loadFileDependencies(text, function (err, depends) {
            if (err) {
                console.log(err); // TODO?
            }
            if (!depends)
                depends = [];
            var workermsg = _this.buildWorkerMessage(depends);
            _this.worker.postMessage(workermsg);
            _this.isCompiling = true;
        });
    };
    CodeProject.prototype.updateFile = function (path, text) {
        this.updateFileInStore(path, text); // TODO: isBinary
        this.filedata[path] = text;
        if (this.okToSend() && this.mainpath) {
            if (this.callbackBuildStatus)
                this.callbackBuildStatus(true);
            this.sendBuild();
        }
    };
    ;
    CodeProject.prototype.setMainFile = function (path) {
        this.mainpath = path;
        if (this.callbackBuildStatus)
            this.callbackBuildStatus(true);
        this.sendBuild();
    };
    CodeProject.prototype.processBuildResult = function (data) {
        // TODO: link listings with source files
        if (data.listings) {
            this.listings = data.listings;
            for (var lstname in this.listings) {
                var lst = this.listings[lstname];
                if (lst.lines)
                    lst.sourcefile = new workertypes_1.SourceFile(lst.lines);
                if (lst.asmlines)
                    lst.assemblyfile = new workertypes_1.SourceFile(lst.asmlines, lst.text);
            }
        }
    };
    CodeProject.prototype.getListings = function () {
        return this.listings;
    };
    // returns first listing in format [prefix].lst (TODO: could be better)
    CodeProject.prototype.getListingForFile = function (path) {
        var fnprefix = util_1.getFilenamePrefix(util_1.getFilenameForPath(path));
        var listings = this.getListings();
        for (var lstfn in listings) {
            if (util_1.getFilenamePrefix(lstfn) == fnprefix) {
                return listings[lstfn];
            }
        }
    };
    return CodeProject;
}());
exports.CodeProject = CodeProject;
//# sourceMappingURL=project.js.map