"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var $ = require("jquery");
var util_1 = require("./util");
var ui_1 = require("./ui");
;
// helper function for editor
function jumpToLine(ed, i) {
    var t = ed.charCoords({ line: i, ch: 0 }, "local").top;
    var middleHeight = ed.getScrollerElement().offsetHeight / 2;
    ed.scrollTo(null, t - middleHeight - 5);
}
// TODO: https://stackoverflow.com/questions/10463518/converting-em-to-px-in-javascript-and-getting-default-font-size
function getVisibleEditorLineHeight() {
    return $("#booksMenuButton").first().height();
}
/////
var MAX_ERRORS = 200;
var SourceEditor = /** @class */ (function () {
    function SourceEditor(path, mode) {
        this.dirtylisting = true;
        this.errormsgs = [];
        this.errorwidgets = [];
        this.path = path;
        this.mode = mode;
    }
    SourceEditor.prototype.createDiv = function (parent, text) {
        var div = document.createElement('div');
        div.setAttribute("class", "editor");
        parent.appendChild(div);
        var asmOverride = text && this.mode == 'verilog' && /__asm\b([\s\S]+?)\b__endasm\b/.test(text);
        this.newEditor(div, asmOverride);
        if (text)
            this.setText(text); // TODO: this calls setCode() and builds... it shouldn't
        this.setupEditor();
        return div;
    };
    SourceEditor.prototype.newEditor = function (parent, isAsmOverride) {
        var isAsm = isAsmOverride || this.mode == '6502' || this.mode == 'z80' || this.mode == 'jsasm' || this.mode == 'gas'; // TODO
        var lineWrap = this.mode == 'markdown';
        this.editor = CodeMirror(parent, {
            theme: 'mbo',
            lineNumbers: true,
            matchBrackets: true,
            tabSize: 8,
            indentAuto: true,
            lineWrapping: lineWrap,
            gutters: isAsm ? ["CodeMirror-linenumbers", "gutter-offset", "gutter-bytes", "gutter-clock", "gutter-info"]
                : ["CodeMirror-linenumbers", "gutter-offset", "gutter-info"],
        });
    };
    SourceEditor.prototype.setupEditor = function () {
        var _this = this;
        var timer;
        this.editor.on('changes', function (ed, changeobj) {
            clearTimeout(timer);
            timer = setTimeout(function () {
                ui_1.current_project.updateFile(_this.path, _this.editor.getValue());
            }, 300);
        });
        this.editor.on('cursorActivity', function (ed) {
            var start = _this.editor.getCursor(true);
            var end = _this.editor.getCursor(false);
            if (start.line == end.line && start.ch < end.ch && end.ch - start.ch < 80) {
                var name = _this.editor.getSelection();
                _this.inspect(name);
            }
            else {
                _this.inspect(null);
            }
        });
        //scrollProfileView(editor);
        this.editor.setOption("mode", this.mode);
    };
    SourceEditor.prototype.inspect = function (ident) {
        var result;
        if (ui_1.platform.inspect) {
            result = ui_1.platform.inspect(ident);
        }
        if (this.inspectWidget) {
            this.inspectWidget.clear();
            this.inspectWidget = null;
        }
        if (result) {
            var infospan = document.createElement("span");
            infospan.setAttribute("class", "tooltipinfoline");
            infospan.appendChild(document.createTextNode(result));
            var line = this.editor.getCursor().line;
            this.inspectWidget = this.editor.addLineWidget(line, infospan, { above: false });
        }
    };
    SourceEditor.prototype.setText = function (text) {
        this.editor.setValue(text); // calls setCode()
        this.editor.clearHistory();
    };
    SourceEditor.prototype.insertText = function (text) {
        var cur = this.editor.getCursor();
        this.editor.replaceRange(text, cur, cur);
    };
    SourceEditor.prototype.getValue = function () {
        return this.editor.getValue();
    };
    SourceEditor.prototype.getPath = function () { return this.path; };
    SourceEditor.prototype.addErrorMarker = function (line, msg) {
        var _this = this;
        var div = document.createElement("div");
        div.setAttribute("class", "tooltipbox tooltiperror");
        div.appendChild(document.createTextNode("\u24cd"));
        this.editor.setGutterMarker(line, "gutter-info", div);
        this.errormsgs.push({ line: line, msg: msg });
        // expand line widgets when mousing over errors
        $(div).mouseover(function (e) {
            _this.expandErrors();
        });
    };
    SourceEditor.prototype.addErrorLine = function (line, msg) {
        var errspan = document.createElement("span");
        errspan.setAttribute("class", "tooltiperrorline");
        errspan.appendChild(document.createTextNode(msg));
        this.errorwidgets.push(this.editor.addLineWidget(line, errspan));
    };
    SourceEditor.prototype.expandErrors = function () {
        var e;
        while (e = this.errormsgs.shift()) {
            this.addErrorLine(e.line, e.msg);
        }
    };
    SourceEditor.prototype.markErrors = function (errors) {
        // TODO: move cursor to error line if offscreen?
        this.clearErrors();
        var numLines = this.editor.lineCount();
        errors = errors.slice(0, MAX_ERRORS);
        for (var _i = 0, errors_1 = errors; _i < errors_1.length; _i++) {
            var info = errors_1[_i];
            // only mark errors with this filename, or without any filename
            if (!info.path || this.path.endsWith(info.path)) {
                var line = info.line - 1;
                if (line < 0 || line >= numLines)
                    line = 0;
                this.addErrorMarker(line, info.msg);
            }
        }
    };
    SourceEditor.prototype.clearErrors = function () {
        this.editor.clearGutter("gutter-info");
        this.refreshDebugState(false);
        this.dirtylisting = true;
        // clear line widgets
        this.errormsgs = [];
        while (this.errorwidgets.length)
            this.errorwidgets.shift().clear();
    };
    SourceEditor.prototype.getSourceFile = function () { return this.sourcefile; };
    SourceEditor.prototype.updateListing = function () {
        // update editor annotations
        // TODO: recreate editor if gutter-bytes is used (verilog)
        this.editor.clearGutter("gutter-info");
        this.editor.clearGutter("gutter-bytes");
        this.editor.clearGutter("gutter-offset");
        this.editor.clearGutter("gutter-clock");
        var lstlines = this.sourcefile.lines || [];
        for (var _i = 0, lstlines_1 = lstlines; _i < lstlines_1.length; _i++) {
            var info = lstlines_1[_i];
            if (info.offset >= 0) {
                this.setGutter("gutter-offset", info.line - 1, util_1.hex(info.offset & 0xffff, 4));
            }
            if (info.insns) {
                var insnstr = info.insns.length > 9 ? ("...") : info.insns;
                this.setGutter("gutter-bytes", info.line - 1, insnstr);
                if (info.iscode) {
                    // TODO: labels trick this part?
                    var opcode = parseInt(info.insns.split(" ")[0], 16);
                    if (ui_1.platform.getOpcodeMetadata) {
                        var meta = ui_1.platform.getOpcodeMetadata(opcode, info.offset);
                        var clockstr = meta.minCycles + "";
                        this.setGutter("gutter-clock", info.line - 1, clockstr);
                    }
                }
            }
        }
    };
    SourceEditor.prototype.setGutter = function (type, line, text) {
        var lineinfo = this.editor.lineInfo(line);
        if (lineinfo && lineinfo.gutterMarkers && lineinfo.gutterMarkers[type]) {
            // do not replace existing marker
        }
        else {
            var textel = document.createTextNode(text);
            this.editor.setGutterMarker(line, type, textel);
        }
    };
    SourceEditor.prototype.setGutterBytes = function (line, s) {
        this.setGutter("gutter-bytes", line - 1, s);
    };
    SourceEditor.prototype.setTimingResult = function (result) {
        this.editor.clearGutter("gutter-bytes");
        // show the lines
        for (var _i = 0, _a = Object.keys(this.sourcefile.line2offset); _i < _a.length; _i++) {
            var line = _a[_i];
            var pc = this.sourcefile.line2offset[line];
            var minclocks = result.pc2minclocks[pc];
            var maxclocks = result.pc2maxclocks[pc];
            if (minclocks >= 0 && maxclocks >= 0) {
                var s;
                if (maxclocks == minclocks)
                    s = minclocks + "";
                else
                    s = minclocks + "-" + maxclocks;
                if (maxclocks == result.MAX_CLOCKS)
                    s += "+";
                this.setGutterBytes(parseInt(line), s);
            }
        }
    };
    SourceEditor.prototype.setCurrentLine = function (line, moveCursor) {
        var _this = this;
        var addCurrentMarker = function (line) {
            var div = document.createElement("div");
            div.style.color = '#66ffff';
            div.appendChild(document.createTextNode("\u25b6"));
            _this.editor.setGutterMarker(line, "gutter-info", div);
        };
        this.clearCurrentLine();
        if (line > 0) {
            addCurrentMarker(line - 1);
            if (moveCursor)
                this.editor.setSelection({ line: line, ch: 0 }, { line: line - 1, ch: 0 }, { scroll: true });
            this.currentDebugLine = line;
        }
    };
    SourceEditor.prototype.clearCurrentLine = function () {
        if (this.currentDebugLine) {
            this.editor.clearGutter("gutter-info");
            this.editor.setSelection(this.editor.getCursor());
            this.currentDebugLine = 0;
        }
    };
    SourceEditor.prototype.getActiveLine = function () {
        var state = ui_1.lastDebugState;
        if (state && state.c && this.sourcefile) {
            var EPC = state.c.EPC || state.c.PC;
            var line = this.sourcefile.findLineForOffset(EPC, 15);
            return line;
        }
        else
            return -1;
    };
    SourceEditor.prototype.refreshDebugState = function (moveCursor) {
        var line = this.getActiveLine();
        if (line >= 0) {
            this.clearCurrentLine();
            this.setCurrentLine(line, moveCursor);
            // TODO: switch to disasm?
        }
    };
    SourceEditor.prototype.refreshListing = function () {
        // lookup corresponding sourcefile for this file, using listing
        var lst = ui_1.current_project.getListingForFile(this.path);
        if (lst && lst.sourcefile && lst.sourcefile !== this.sourcefile) {
            this.sourcefile = lst.sourcefile;
            this.dirtylisting = true;
        }
        if (!this.sourcefile || !this.dirtylisting)
            return;
        this.dirtylisting = false;
        this.updateListing();
    };
    SourceEditor.prototype.refresh = function (moveCursor) {
        this.refreshListing();
        this.refreshDebugState(moveCursor);
    };
    SourceEditor.prototype.getLine = function (line) {
        return this.editor.getLine(line - 1);
    };
    SourceEditor.prototype.getCurrentLine = function () {
        return this.editor.getCursor().line + 1;
    };
    SourceEditor.prototype.getCursorPC = function () {
        var line = this.getCurrentLine();
        while (this.sourcefile && line >= 0) {
            var pc = this.sourcefile.line2offset[line];
            if (pc >= 0)
                return pc;
            line--;
        }
        return -1;
    };
    // bitmap editor (TODO: refactor)
    SourceEditor.prototype.openBitmapEditorWithParams = function (fmt, bytestr, palfmt, palstr) {
        var _this = this;
        var handleWindowMessage = function (e) {
            //console.log("window message", e.data);
            if (e.data.bytes) {
                _this.editor.replaceSelection(e.data.bytestr);
            }
            if (e.data.close) {
                $("#pixeditback").hide();
            }
            e.target.removeEventListener("message", handleWindowMessage);
        };
        $("#pixeditback").show();
        window.addEventListener("message", handleWindowMessage, false); // TODO: remove listener
        window['pixeditframe'].contentWindow.postMessage({ fmt: fmt, bytestr: bytestr, palfmt: palfmt, palstr: palstr }, '*');
    };
    SourceEditor.prototype.lookBackwardsForJSONComment = function (line, req) {
        var re = /[/;][*;]([{].+[}])[*;][/;]/;
        while (--line >= 0) {
            var s = this.editor.getLine(line);
            var m = re.exec(s);
            if (m) {
                var jsontxt = m[1].replace(/([A-Za-z]+):/g, '"$1":'); // fix lenient JSON
                var obj = JSON.parse(jsontxt);
                if (obj[req]) {
                    var start = { obj: obj, line: line, ch: s.indexOf(m[0]) + m[0].length };
                    var line0 = line;
                    var pos0 = start.ch;
                    line--;
                    while (++line < this.editor.lineCount()) {
                        var l = this.editor.getLine(line);
                        var endsection;
                        if (ui_1.platform_id == 'verilog')
                            endsection = l.indexOf('end') >= pos0;
                        else if (s.startsWith(';;'))
                            endsection = l.indexOf(';;') >= pos0;
                        else
                            endsection = l.indexOf(';') >= pos0;
                        if (endsection) {
                            var end = { line: line, ch: this.editor.getLine(line).length };
                            return { obj: obj, start: start, end: end };
                        }
                        pos0 = 0;
                    }
                    line = line0;
                }
            }
        }
    };
    SourceEditor.prototype.openBitmapEditorAtCursor = function () {
        if ($("#pixeditback").is(":visible")) {
            $("#pixeditback").hide(250);
            return;
        }
        var line = this.editor.getCursor().line + 1;
        var data = this.lookBackwardsForJSONComment(this.getCurrentLine(), 'w');
        if (data && data.obj && data.obj.w > 0 && data.obj.h > 0) {
            var paldata = this.lookBackwardsForJSONComment(data.start.line - 1, 'pal');
            var palbytestr;
            if (paldata) {
                palbytestr = this.editor.getRange(paldata.start, paldata.end);
                paldata = paldata.obj;
            }
            this.editor.setSelection(data.end, data.start);
            this.openBitmapEditorWithParams(data.obj, this.editor.getSelection(), paldata, palbytestr);
        }
        else {
            alert("To edit graphics, move cursor to a constant array preceded by a comment in the format:\n\n/*{w:,h:,bpp:,count:...}*/\n\n(See code examples)");
        }
    };
    return SourceEditor;
}());
exports.SourceEditor = SourceEditor;
///
var DisassemblerView = /** @class */ (function () {
    function DisassemblerView() {
    }
    DisassemblerView.prototype.getDisasmView = function () { return this.disasmview; };
    DisassemblerView.prototype.createDiv = function (parent) {
        var div = document.createElement('div');
        div.setAttribute("class", "editor");
        parent.appendChild(div);
        this.newEditor(div);
        return div;
    };
    DisassemblerView.prototype.newEditor = function (parent) {
        this.disasmview = CodeMirror(parent, {
            mode: 'z80',
            theme: 'cobalt',
            tabSize: 8,
            readOnly: true,
            styleActiveLine: true
        });
    };
    // TODO: too many globals
    DisassemblerView.prototype.refresh = function (moveCursor) {
        var state = ui_1.lastDebugState || ui_1.platform.saveState();
        var pc = state.c ? state.c.PC : 0;
        var curline = 0;
        var selline = 0;
        var addr2symbol = (ui_1.platform.debugSymbols && ui_1.platform.debugSymbols.addr2symbol) || {};
        // TODO: not perfect disassembler
        var disassemble = function (start, end) {
            if (start < 0)
                start = 0;
            if (end > 0xffff)
                end = 0xffff;
            // TODO: use pc2visits
            var a = start;
            var s = "";
            while (a < end) {
                var disasm = ui_1.platform.disassemble(a, ui_1.platform.readAddress.bind(ui_1.platform));
                /* TODO: look thru all source files
                var srclinenum = sourcefile && this.sourcefile.offset2line[a];
                if (srclinenum) {
                  var srcline = getActiveEditor().getLine(srclinenum);
                  if (srcline && srcline.trim().length) {
                    s += "; " + srclinenum + ":\t" + srcline + "\n";
                    curline++;
                  }
                }
                */
                var bytes = "";
                var comment = "";
                for (var i = 0; i < disasm.nbytes; i++)
                    bytes += util_1.hex(ui_1.platform.readAddress(a + i));
                while (bytes.length < 14)
                    bytes += ' ';
                var dstr = disasm.line;
                if (addr2symbol && disasm.isaddr) {
                    dstr = dstr.replace(/([^#])[$]([0-9A-F]+)/, function (substr) {
                        var args = [];
                        for (var _i = 1; _i < arguments.length; _i++) {
                            args[_i - 1] = arguments[_i];
                        }
                        var addr = parseInt(args[1], 16);
                        var sym = addr2symbol[addr];
                        if (sym)
                            return (args[0] + sym);
                        sym = addr2symbol[addr - 1];
                        if (sym)
                            return (args[0] + sym + "+1");
                        return substr;
                    });
                }
                if (addr2symbol) {
                    var sym = addr2symbol[a];
                    if (sym) {
                        comment = "; " + sym;
                    }
                }
                var dline = util_1.hex(parseInt(a), 4) + "\t" + util_1.rpad(bytes, 14) + "\t" + util_1.rpad(dstr, 30) + comment + "\n";
                s += dline;
                if (a == pc)
                    selline = curline;
                curline++;
                a += disasm.nbytes || 1;
            }
            return s;
        };
        var text = disassemble(pc - 96, pc) + disassemble(pc, pc + 96);
        this.disasmview.setValue(text);
        if (moveCursor) {
            this.disasmview.setCursor(selline, 0);
            jumpToLine(this.disasmview, selline);
        }
    };
    DisassemblerView.prototype.getCursorPC = function () {
        var line = this.disasmview.getCursor().line;
        if (line >= 0) {
            var toks = this.disasmview.getLine(line).trim().split(/\s+/);
            if (toks && toks.length >= 1) {
                var pc = parseInt(toks[0], 16);
                if (pc >= 0)
                    return pc;
            }
        }
        return -1;
    };
    return DisassemblerView;
}());
exports.DisassemblerView = DisassemblerView;
///
var ListingView = /** @class */ (function (_super) {
    __extends(ListingView, _super);
    function ListingView(lstfn) {
        var _this = _super.call(this) || this;
        _this.path = lstfn;
        return _this;
    }
    ListingView.prototype.refreshListing = function () {
        // lookup corresponding assemblyfile for this file, using listing
        var lst = ui_1.current_project.getListingForFile(this.path);
        if (lst && lst.assemblyfile && lst.assemblyfile !== this.assemblyfile) {
            this.assemblyfile = lst.assemblyfile;
        }
    };
    ListingView.prototype.refresh = function (moveCursor) {
        this.refreshListing();
        if (!this.assemblyfile)
            return; // TODO?
        var state = ui_1.lastDebugState || ui_1.platform.saveState();
        var pc = state.c ? (state.c.EPC || state.c.PC) : 0;
        var asmtext = this.assemblyfile.text;
        var disasmview = this.getDisasmView();
        disasmview.setValue(asmtext);
        var debugging = ui_1.platform.isDebugging && ui_1.platform.isDebugging();
        var findPC = debugging ? pc : -1;
        if (findPC >= 0 && this.assemblyfile) {
            var lineno = this.assemblyfile.findLineForOffset(findPC, 15);
            if (lineno && moveCursor) {
                // set cursor while debugging
                if (debugging)
                    disasmview.setCursor(lineno - 1, 0);
                jumpToLine(disasmview, lineno - 1);
            }
        }
    };
    return ListingView;
}(DisassemblerView));
exports.ListingView = ListingView;
///
// TODO: make it use debug state
// TODO: make it safe (load/restore state?)
var MemoryView = /** @class */ (function () {
    function MemoryView() {
    }
    /*
    read(addr:number) {
      // TODO: b offset ?
      if (lastDebugState && lastDebugState.b && addr < lastDebugState.b.length)
        return lastDebugState.b[addr];
      else
        return this.platform.readMemory(addr);
    }
    */
    MemoryView.prototype.createDiv = function (parent) {
        var div = document.createElement('div');
        div.setAttribute("class", "memdump");
        parent.appendChild(div);
        this.showMemoryWindow(parent, div);
        return this.maindiv = div;
    };
    MemoryView.prototype.showMemoryWindow = function (workspace, parent) {
        var _this = this;
        this.memorylist = new VirtualList({
            w: $(workspace).width(),
            h: $(workspace).height(),
            itemHeight: getVisibleEditorLineHeight(),
            totalRows: 0x2000,
            generatorFn: function (row) {
                var s = _this.getMemoryLineAt(row);
                var linediv = document.createElement("div");
                if (_this.dumplines) {
                    var dlr = _this.dumplines[row];
                    if (dlr)
                        linediv.classList.add('seg_' + _this.getMemorySegment(_this.dumplines[row].a));
                }
                linediv.appendChild(document.createTextNode(s));
                return linediv;
            }
        });
        $(parent).append(this.memorylist.container);
        this.tick();
        if (ui_1.compparams && this.dumplines)
            this.memorylist.scrollToItem(this.findMemoryWindowLine(ui_1.compparams.data_start));
    };
    MemoryView.prototype.refresh = function () {
        this.dumplines = null;
        this.tick();
    };
    MemoryView.prototype.tick = function () {
        var _this = this;
        if (this.memorylist) {
            $(this.maindiv).find('[data-index]').each(function (i, e) {
                var div = $(e);
                var row = parseInt(div.attr('data-index'));
                var oldtext = div.text();
                var newtext = _this.getMemoryLineAt(row);
                if (oldtext != newtext)
                    div.text(newtext);
            });
        }
    };
    MemoryView.prototype.getMemoryLineAt = function (row) {
        var offset = row * 16;
        var n1 = 0;
        var n2 = 16;
        var sym;
        if (this.getDumpLines()) {
            var dl = this.dumplines[row];
            if (dl) {
                offset = dl.a & 0xfff0;
                n1 = dl.a - offset;
                n2 = n1 + dl.l;
                sym = dl.s;
            }
            else {
                return '.';
            }
        }
        var s = util_1.hex(offset + n1, 4) + ' ';
        for (var i = 0; i < n1; i++)
            s += '   ';
        if (n1 > 8)
            s += ' ';
        for (var i = n1; i < n2; i++) {
            var read = ui_1.platform.readAddress(offset + i);
            if (i == 8)
                s += ' ';
            s += ' ' + (read !== null ? util_1.hex(read, 2) : '??');
        }
        for (var i = n2; i < 16; i++)
            s += '   ';
        if (sym)
            s += '  ' + sym;
        return s;
    };
    MemoryView.prototype.getDumpLineAt = function (line) {
        var d = this.dumplines[line];
        if (d) {
            return d.a + " " + d.s;
        }
    };
    // TODO: addr2symbol for ca65; and make it work without symbols
    MemoryView.prototype.getDumpLines = function () {
        var addr2sym = (ui_1.platform.debugSymbols && ui_1.platform.debugSymbols.addr2symbol) || {};
        if (!this.dumplines) {
            this.dumplines = [];
            var ofs = 0;
            var sym;
            for (var _i = 0, _a = Object.keys(addr2sym); _i < _a.length; _i++) {
                var _nextofs = _a[_i];
                var nextofs = parseInt(_nextofs); // convert from string (stupid JS)
                var nextsym = addr2sym[nextofs];
                if (sym) {
                    if (MemoryView.IGNORE_SYMS[sym]) {
                        ofs = nextofs;
                    }
                    else {
                        while (ofs < nextofs) {
                            var ofs2 = (ofs + 16) & 0xffff0;
                            if (ofs2 > nextofs)
                                ofs2 = nextofs;
                            //if (ofs < 1000) console.log(ofs, ofs2, nextofs, sym);
                            this.dumplines.push({ a: ofs, l: ofs2 - ofs, s: sym });
                            ofs = ofs2;
                        }
                    }
                }
                sym = nextsym;
            }
        }
        return this.dumplines;
    };
    MemoryView.prototype.getMemorySegment = function (a) {
        if (!ui_1.compparams)
            return 'unknown';
        if (a >= ui_1.compparams.data_start && a < ui_1.compparams.data_start + ui_1.compparams.data_size) {
            if (ui_1.platform.getSP && a >= ui_1.platform.getSP() - 15)
                return 'stack';
            else
                return 'data';
        }
        else if (a >= ui_1.compparams.code_start && a < ui_1.compparams.code_start + (ui_1.compparams.code_size || ui_1.compparams.rom_size))
            return 'code';
        else
            return 'unknown';
    };
    MemoryView.prototype.findMemoryWindowLine = function (a) {
        for (var i = 0; i < this.dumplines.length; i++)
            if (this.dumplines[i].a >= a)
                return i;
    };
    MemoryView.IGNORE_SYMS = { s__INITIALIZER: true, /* s__GSINIT:true, */ _color_prom: true };
    return MemoryView;
}());
exports.MemoryView = MemoryView;
///
var BinaryFileView = /** @class */ (function () {
    function BinaryFileView(path, data) {
        this.path = path;
        this.data = data;
    }
    BinaryFileView.prototype.createDiv = function (parent) {
        var div = document.createElement('div');
        div.setAttribute("class", "memdump");
        parent.appendChild(div);
        this.showMemoryWindow(parent, div);
        return this.maindiv = div;
    };
    BinaryFileView.prototype.showMemoryWindow = function (workspace, parent) {
        var _this = this;
        this.memorylist = new VirtualList({
            w: $(workspace).width(),
            h: $(workspace).height(),
            itemHeight: getVisibleEditorLineHeight(),
            totalRows: ((this.data.length + 15) >> 4),
            generatorFn: function (row) {
                var s = _this.getMemoryLineAt(row);
                var linediv = document.createElement("div");
                linediv.appendChild(document.createTextNode(s));
                return linediv;
            }
        });
        $(parent).append(this.memorylist.container);
    };
    BinaryFileView.prototype.getMemoryLineAt = function (row) {
        var offset = row * 16;
        var n1 = 0;
        var n2 = 16;
        var s = util_1.hex(offset + n1, 4) + ' ';
        for (var i = 0; i < n1; i++)
            s += '   ';
        if (n1 > 8)
            s += ' ';
        for (var i = n1; i < n2; i++) {
            var read = this.data[offset + i];
            if (i == 8)
                s += ' ';
            s += ' ' + (read >= 0 ? util_1.hex(read, 2) : '  ');
        }
        return s;
    };
    BinaryFileView.prototype.refresh = function () {
    };
    BinaryFileView.prototype.getPath = function () { return this.path; };
    return BinaryFileView;
}());
exports.BinaryFileView = BinaryFileView;
//# sourceMappingURL=views.js.map